@@ -29,7 +29,6 @@
 (require 'chess-input)
 (require 'chess-message)
 (require 'chess-module)
-(require 'chess-pgn)
 (require 'chess-random)
 (require 'chess-var)
 
@@ -352,7 +351,14 @@ also view the same game."
 		  (point-min))))))
     (aref chess-display-index-positions index)))
 
-(defun chess-display-draw-square (display index piece &optional pos)
+(defun chess-display-draw-square (display index &optional piece pos)
+  "(Re)draw the square of DISPLAY indicated by INDEX.
+Optional argument PIECE indicates the piece (or blank) to draw.
+If it is not provided, `chess-display-position' is consulted.
+Optional argument POS indicates the buffer position to draw the square at.
+If that is not provided, `chess-display-index-pos' is called.
+
+This function is especially useful to clear a previously set highlight."
   (cl-check-type display (or null buffer))
   (cl-check-type index (integer 0 63))
   (cl-check-type piece (member nil ?  ?P ?N ?B ?R ?Q ?K ?p ?n ?b ?r ?q ?k))
@@ -396,35 +402,38 @@ also view the same game."
   "Return non-nil if the displayed chessboard reflects an active game.
 Basically, it means we are playing, not editing or reviewing."
   (and (chess-game-data chess-module-game 'active)
-       (= chess-display-index
-	  (chess-game-index chess-module-game))
+       (= chess-display-index (chess-game-index chess-module-game))
        (not (chess-game-over-p chess-module-game))
        (not chess-display-edit-mode)))
 
 (defun chess-display-move (display ply)
   "Move a piece on DISPLAY, by applying the given PLY.
-The position of PLY must match the currently displayed position."
+The position of PLY must match the currently displayed position.
+
+This adds PLY to the game associated with DISPLAY."
   (chess-with-current-buffer display
-    (if (and (chess-display-active-p)
-	     ;; `active' means we're playing against an engine
-	     (chess-game-data chess-module-game 'active)
-	     (not (eq (chess-game-data chess-module-game 'my-color)
-		      (chess-game-side-to-move chess-module-game))))
-	(chess-error 'not-your-move)
-      (if (and (= chess-display-index
-		  (chess-game-index chess-module-game))
-	       (chess-game-over-p chess-module-game))
-	  (chess-error 'game-is-over)))
-    (if (= chess-display-index (chess-game-index chess-module-game))
-	(let ((chess-display-handling-event t))
-	  (chess-game-move chess-module-game ply)
-	  (chess-display-paint-move nil ply)
-	  (chess-display-set-index* nil (chess-game-index chess-module-game))
-	  (redisplay)                   ; FIXME: This is clearly necessary, but why?
-	  (chess-game-run-hooks chess-module-game 'post-move))
-      ;; jww (2002-03-28): This should beget a variation within the
-      ;; game, or alter the game, just as SCID allows
-      (chess-error 'cannot-yet-add))))
+    (cond ((and (chess-display-active-p)
+		;; `active' means we're playing against an engine
+		(chess-game-data chess-module-game 'active)
+		(not (eq (chess-game-data chess-module-game 'my-color)
+			 (chess-game-side-to-move chess-module-game))))
+	   (chess-error 'not-your-move))
+
+	  ((and (= chess-display-index (chess-game-index chess-module-game))
+		(chess-game-over-p chess-module-game))
+	   (chess-error 'game-is-over))
+
+	  ((= chess-display-index (chess-game-index chess-module-game))
+	   (let ((chess-display-handling-event t))
+	     (chess-game-move chess-module-game ply)
+	     (chess-display-paint-move nil ply)
+	     (chess-display-set-index* nil (chess-game-index chess-module-game))
+	     (redisplay)                   ; FIXME: This is clearly necessary, but why?
+	     (chess-game-run-hooks chess-module-game 'post-move)))
+
+	  (t ;; jww (2002-03-28): This should beget a variation within the
+	     ;; game, or alter the game, just as SCID allows
+	     (chess-error 'cannot-yet-add)))))
 
 (defun chess-display-highlight (display &rest args)
   "Highlight the square at INDEX on the current position.
@@ -714,6 +723,8 @@ The key bindings available in this mode are:
   (interactive "sSet from FEN string: ")
   (chess-display-set-position nil (chess-fen-to-pos fen)))
 
+(declare-function chess-game-to-pgn "chess-pgn" (game &optional indented to-string))
+
 (defun chess-display-kill-board (&optional arg)
   "Send the current board configuration to the user."
   (interactive "P")
@@ -725,6 +736,8 @@ The key bindings available in this mode are:
 		    (buffer-string)))
       (kill-new (chess-pos-to-fen (chess-display-position nil) t)))))
 
+(declare-function chess-pgn-to-game "chess-pgn" (&optional string))
+
 (defun chess-display-yank-board ()
   "Send the current board configuration to the user."
   (interactive)
@@ -1042,7 +1055,7 @@ to the end or beginning."
 
     (define-key map [(control ?l)] 'chess-display-redraw)
     (define-key map [(control ?i)] 'chess-display-invert)
-    (define-key map [tab] 'chess-display-invert)
+    (define-key map "\t" 'chess-display-invert)
 
     (define-key map [??] 'describe-mode)
     (define-key map [?L] 'chess-display-list-buffers)
@@ -1106,7 +1119,7 @@ to the end or beginning."
 				 chess-display-edit-position))
 
 (defun chess-display-restore-board ()
-  "Setup the current board for editing."
+  "Cancel editing."
   (interactive)
   (chess-display-end-edit-mode)
   ;; reset the modeline
@@ -1230,7 +1243,7 @@ Clicking once on a piece selects it; then click on the target location."
 		    (setq chess-display-last-selected nil))
 		(let ((piece (chess-pos-piece position coord)))
 		  (cond
-		   ((eq piece ? )
+		   ((= piece ? )
 		    (throw 'message (chess-string 'selected-empty)))
 		   ((not (or chess-display-edit-mode
 			     (not (chess-display-active-p))
