@@ -48,8 +48,13 @@
 (make-variable-buffer-local 'chess-input-position-function)
 (make-variable-buffer-local 'chess-input-move-function)
 
+(defgroup chess-input nil
+  "Move input related otpions."
+  :group 'chess)
+
 (defcustom chess-input-notation-type :san
   "Define the notation type to use for move input."
+  :group 'chess-input
   :type '(choice (const :tag "Standard (short) algebraic notation" :san)
 		 (const :tag "Numeric notation" :numeric)))
 
@@ -59,21 +64,26 @@
 	 (i 0) (x 0) (l (length move))
 	 (xl (length chess-input-move-string)))
     (unless (or (and (equal (downcase chess-input-move-string) "ok")
-		     (string-match "\\`O-O[+#]?\\'" move))
+		     (chess-ply-keyword ply :castle))
 		(and (equal (downcase chess-input-move-string) "oq")
-		     (string-match "\\`O-O-O[+#]?\\'" move)))
+		     (chess-ply-keyword ply :long-castle)))
       (while (and (< i l) (< x xl))
 	(let ((move-char (aref move i))
 	      (entry-char (aref chess-input-move-string x)))
-	  (cond
-	   ((or (and (= move-char ?x) (/= entry-char ?x))
-		(and (= move-char ?=) (/= entry-char ?=)))
-	    (setq i (1+ i)))
-	   ((/= entry-char (if (< entry-char ?a) move-char (downcase move-char)))
-	    (setq ply nil i l))
-	   (t (setq i (1+ i) x (1+ x)))))))
+	  (cond ((or (and (= move-char ?x) (/= entry-char ?x))
+		     (and (= move-char ?=) (/= entry-char ?=)))
+		 (setq i (1+ i)))
+		((/= entry-char (if (< entry-char ?a)
+				    move-char
+				  (downcase move-char)))
+		 (setq ply nil i l))
+		(t (setq i (1+ i) x (1+ x)))))))
     ply))
 
+(defvar chess-display-highlight-legal nil)
+(declare-function chess-display-redraw "chess-display" (&optional display))
+(declare-function chess-display-highlight "chess-display" (display &rest args))
+
 (defun chess-input-display-moves (&optional move-list)
   (unless move-list
     (setq move-list
@@ -83,7 +93,7 @@
   (when (> (length chess-input-move-string) 0)
     (when chess-display-highlight-legal
       (apply #'chess-display-highlight
-	     nil (cl-delete-duplicates (mapcar #'chess-ply-target move-list))))
+	     nil (delete-dups (mapcar #'chess-ply-target move-list))))
     (message "[%s] %s" chess-input-move-string
 	     (mapconcat (lambda (ply)
 			  (chess-ply-to-algebraic ply chess-input-notation-type))
@@ -118,63 +128,54 @@
 		     ?k
 		   last-command-event))
       (if (or (memq (upcase char) '(?K ?Q ?N ?B ?R ?P))
-	      (and (>= char ?a) (<= char ?h)))
+	      (and (>= char ?a) (<= char ?h))
+	      (and (>= char ?1) (<= char ?8)))
 	  (setq chess-input-moves-pos position
 		chess-input-moves
 		(cons
 		 char
 		 (sort
-		  (if (eq char ?b)
-		      (append (chess-legal-plies
-			       position :piece (if color ?P ?p) :file 1)
-			      (chess-legal-plies
-			       position :piece (if color ?B ?b)))
-		    (if (and (>= char ?a)
-			     (<= char ?h))
-			(chess-legal-plies position
-					   :piece (if color ?P ?p)
-					   :file (- char ?a))
-		      (chess-legal-plies position
-					 :piece (if color
-						    (upcase char)
-						  (downcase char)))))
-		  (function
-		   (lambda (left right)
-		     (string-lessp (chess-ply-to-algebraic left)
-				   (chess-ply-to-algebraic right)))))))
-	(if (and (>= char ?1) (<= char ?8))
-	    (setq chess-input-moves-pos position
-		  chess-input-moves
-		  (cons
-		   char
-		   (sort
-		    (chess-legal-plies position :color color :file (- char ?1))
-		    (function
-		     (lambda (left right)
-		       (string-lessp (chess-ply-to-algebraic left)
-				     (chess-ply-to-algebraic right)))))))))))
-  (let ((moves (delq nil (mapcar 'chess-input-test-move
+		  (cond ((eq char ?b)
+			 (nconc (chess-legal-plies
+				 position :piece (if color ?P ?p) :file 1)
+				(chess-legal-plies
+				 position :piece (if color ?B ?b))))
+			((and (>= char ?a) (<= char ?h))
+			 (chess-legal-plies
+			  position :piece (if color ?P ?p)
+			  :file (chess-file-from-char char)))
+			((and (>= char ?1) (<= char ?8))
+			 (chess-legal-plies
+			  position :color color :file (- char ?1)))
+			(t (chess-legal-plies
+			    position :piece (if color
+						(upcase char)
+					      (downcase char)))))
+		  (lambda (left right)
+		    (string-lessp (chess-ply-to-algebraic left)
+				  (chess-ply-to-algebraic right)))))))))
+  (let ((moves (delq nil (mapcar #'chess-input-test-move
 				 (cdr chess-input-moves)))))
-    (cond
-     ((or (= (length moves) 1)
-	  ;; if there is an exact match except for case, it must be an
-	  ;; abiguity between a bishop and a b-pawn move.  In this
-	  ;; case, always take the b-pawn move; to select the bishop
-	  ;; move, use B to begin the keyboard shortcut
-	  (and (= (length moves) 2)
-	       (string= (downcase (chess-ply-to-algebraic (car moves)))
-			(downcase (chess-ply-to-algebraic (cadr moves))))
-	       (setq moves (cdr moves))))
-      (funcall chess-input-move-function nil (car moves))
-      (when chess-display-highlight-legal
-	(chess-display-redraw nil))
-      (setq chess-input-move-string nil
-	    chess-input-moves nil
-	    chess-input-moves-pos nil))
-     ((null moves)
-      (chess-input-shortcut-delete))
-     (t
-      (chess-input-display-moves moves)))))
+    (cond ((or (= (length moves) 1)
+	       ;; if there is an exact match except for case, it must be an
+	       ;; abiguity between a bishop and a b-pawn move.  In this
+	       ;; case, always take the b-pawn move; to select the bishop
+	       ;; move, use B to begin the keyboard shortcut
+	       (and (= (length moves) 2)
+		    (string= (downcase (chess-ply-to-algebraic (car moves)))
+			     (downcase (chess-ply-to-algebraic (cadr moves))))
+		    (setq moves (cdr moves))))
+	   (funcall chess-input-move-function nil (car moves))
+	   (when chess-display-highlight-legal
+	     (chess-display-redraw nil))
+	   (setq chess-input-move-string nil
+		 chess-input-moves nil
+		 chess-input-moves-pos nil))
+
+	  ((null moves)
+	   (chess-input-shortcut-delete))
+
+	  (t (chess-input-display-moves moves)))))
 
 (provide 'chess-input)
 
