@@ -54,7 +54,6 @@
 
 ;;; Code:
 
-(require 'chess)
 (require 'chess-algebraic)
 (require 'chess-display)
 (require 'chess-fen)
@@ -93,6 +92,7 @@
 	    (unless ply
 	      (chess-error 'pgn-read-error move))
 	    (setq position (chess-ply-next-pos ply))
+	    (chess-pos-set-annotations position nil)
 	    (nconc plies (list ply))))
 
 	 ((and top-level
@@ -281,6 +281,8 @@ PGN text."
 (chess-message-catalog 'english
   '((could-not-read-pgn . "Could not read or find a PGN game")))
 
+(declare-function chess-create-display "chess.el" (perspective &optional modules-too))
+
 ;;;###autoload
 (defun chess-pgn-read (&optional file)
   "Read and display a PGN game after point."
