@@ -287,7 +287,7 @@ color will do."
   (cl-check-type piece-or-color (member t nil ?  ?K ?Q ?N ?B ?R ?P ?k ?q ?n ?b ?r ?p))
   (let ((p (chess-pos-piece position index)))
     (cond
-     ((= p ? ) (eq p piece-or-color))
+     ((= p ? ) (and (numberp piece-or-color) (= p piece-or-color)))
      ((eq piece-or-color t) (< p ?a))
      ((eq piece-or-color nil) (> p ?a))
      (t (= p piece-or-color)))))
@@ -308,16 +308,34 @@ color will do."
   (cl-check-type index (integer 0 63))
   (mod index 8))
 
+(defsubst chess-rank-to-char (rank)
+  (cl-check-type rank (integer 0 7))
+  (+ (- 7 rank) ?1))
+
+(defsubst chess-rank-from-char (character)
+  (cl-check-type character character)
+  (- 7 (- character ?1)))
+
+(defsubst chess-file-to-char (file)
+  (cl-check-type file (integer 0 7))
+  (+ file ?a))
+
+(defsubst chess-file-from-char (character)
+  (cl-check-type character character)
+  (- character ?a))
+
 (defsubst chess-coord-to-index (coord)
   "Convert a COORD string (such as \"e4\" into an index value."
   (cl-check-type coord string)
   (cl-assert (= (length coord) 2))
-  (chess-rf-to-index (- 7 (- (aref coord 1) ?1)) (- (aref coord 0) ?a)))
+  (chess-rf-to-index (chess-rank-from-char (aref coord 1))
+		     (chess-file-from-char (aref coord 0))))
 
 (defsubst chess-index-to-coord (index)
   "Convert the chess position INDEX into a coord string."
   (cl-check-type index (integer 0 63))
-  (string (+ (chess-index-file index) ?a) (+ (- 7 (chess-index-rank index)) ?1)))
+  (string (chess-file-to-char (chess-index-file index))
+	  (chess-rank-to-char (chess-index-rank index))))
 
 (defsubst chess-incr-index (index rank-move file-move)
   "Create a new INDEX from an old one, by adding RANK-MOVE and FILE-MOVE."
@@ -486,12 +504,12 @@ alist, but the `cdr' of their entries will be nil."
 			  (when ok
 			    (memq piece '(?P ?N ?B ?R ?Q ?K ?p ?n ?b ?r ?q ?k))))
 			pieces :initial-value t))
-  (cl-assert (= (length pieces) (length (cl-delete-duplicates pieces))))
+  (cl-assert (equal pieces (delete-dups pieces)))
   (let ((alist (mapcar #'list pieces)))
     (dotimes (index 64)
       (let ((piece (chess-pos-piece position index)))
-	(unless (eq piece ? )
-	  (let ((entry (assq piece alist)))
+	(unless (= piece ? )
+	  (let ((entry (assoc piece alist)))
 	    (when entry (push index (cdr entry)))))))
     alist))
 
@@ -693,8 +711,8 @@ The current side-to-move is always white."
 
 (defun chess-pos-material-value (position color)
   "Return the aggregate material value in POSITION for COLOR."
-  (cl-assert (vectorp position))
-  (cl-assert (memq color '(nil t)))
+  (cl-check-type position chess-pos)
+  (cl-check-type color (member nil t))
   (let ((pieces (chess-pos-search position color))
 	(value 0))
     (dolist (index pieces)
@@ -889,9 +907,9 @@ indices which indicate where a piece may have moved from.
 If CHECK-ONLY is non-nil and PIECE is either t or nil, only consider
 pieces which can give check (not the opponents king).
 If NO-CASTLING is non-nil, do not consider castling moves."
-  (cl-assert (vectorp position))
-  (cl-assert (and (>= target 0) (< target 64)))
-  (cl-assert (memq piece '(t nil ?K ?Q ?N ?B ?R ?P ?k ?q ?n ?b ?r ?p)))
+  (cl-check-type position chess-pos)
+  (cl-check-type target (integer 0 63))
+  (cl-check-type piece (member t nil ?K ?Q ?N ?B ?R ?P ?k ?q ?n ?b ?r ?p))
   (let* ((color (if (characterp piece)
 		    (< piece ?a)
 		  piece))
@@ -911,7 +929,7 @@ If NO-CASTLING is non-nil, do not consider castling moves."
 	  (let ((pos-piece (chess-pos-piece position (caar ray))))
 	    (setq ray (cond ((memq pos-piece (cdar ray))
 			     (chess--add-candidate (caar ray)) nil)
-			    ((eq pos-piece ? ) (cdr ray)))))))
+			    ((= pos-piece ? ) (cdr ray)))))))
 
       ;; test for knights and pawns
       (dolist (p (if piece '(?P ?N) '(?p ?n)))
@@ -943,10 +961,10 @@ If NO-CASTLING is non-nil, do not consider castling moves."
 
      ;; pawn movement, which is diagonal 1 when taking, but forward
      ;; 1 or 2 when moving (the most complex piece, actually)
-     ((eq test-piece ?P)
+     ((= test-piece ?P)
       (let ((p (chess-pos-piece position target))
 	    (backward (if color chess-direction-south chess-direction-north)))
-	(if (if (eq p ? )
+	(if (if (= p ? )
 		;; check for en passant
 		(and (= (chess-index-rank target) (if color 2 5))
 		     (let ((ep (chess-pos-en-passant position)))
@@ -978,9 +996,9 @@ If NO-CASTLING is non-nil, do not consider castling moves."
 		  (chess--add-candidate pos)))
 	  (if (setq pos (chess-next-index target backward))
 	      (let ((pos-piece (chess-pos-piece position pos)))
-		(if (eq pos-piece piece)
+		(if (= pos-piece piece)
 		    (chess--add-candidate pos)
-		  (if (and (eq pos-piece ? )
+		  (if (and (= pos-piece ? )
 			   (= (if color 4 3) (chess-index-rank target))
 			   (setq pos (funcall (if color #'+ #'-) pos 8))
 			   (chess-pos-piece-p position pos piece))
@@ -997,11 +1015,11 @@ If NO-CASTLING is non-nil, do not consider castling moves."
 	(setq pos (chess-next-index target dir))
 	(while pos
 	  (let ((pos-piece (chess-pos-piece position pos)))
-	    (if (eq pos-piece piece)
+	    (if (= pos-piece piece)
 		(progn
 		  (chess--add-candidate pos)
 		  (setq pos nil))
-	      (setq pos (and (eq pos-piece ? ) (chess-next-index pos dir)))))))
+	      (setq pos (and (= pos-piece ? ) (chess-next-index pos dir)))))))
       ;; test whether the rook can move to the target by castling
       (if (and (= test-piece ?R) (not no-castling))
 	  (let (rook)
